"""Parser for Split AC (009-199) device type."""
from typing import Dict

from .base import BaseDeviceParser, DeviceAttribute

class SplitWater035699Parser(BaseDeviceParser):
    """Parser for Split AC 009-199 device type."""
    
    @property
    def device_type(self) -> str:
        return "035"
        
    @property
    def feature_code(self) -> str:
        return "699"
        
    @property
    def attributes(self) -> Dict[str, DeviceAttribute]:
        return {
            "t_work_mode": DeviceAttribute(
                key="t_work_mode",
                name="设定模式",
                attr_type="Enum",
                step=1,
                value_range="1, 0, 15, 5, 6, 16, 3",
                value_map={
                    "0": "制热",
                    "1": "制冷",
                    "15": "自动",
                    "5": "热水+制冷",
                    "16": "热水+自动",
                    "3": "热水",
                    "6": "热水+制热"
                },
                read_write="RW"
            ),
            "t_temp": DeviceAttribute(
                key="t_temp",
                name="设置温度",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="RW"
            ),
            "t_power": DeviceAttribute(
                key="t_power",
                name="开关机",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "关",
                    "1": "开"
                },
                read_write="RW"
            ),
            "f_zone1water_temp1": DeviceAttribute(
                key="f_zone1water_temp1",
                name="1温区实际值",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="R"
            ),
            "f_zone2water_temp2": DeviceAttribute(
                key="f_zone2water_temp2",
                name="2温区实际值",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="R"
            ),
            "f_water_tank_temp": DeviceAttribute(
                key="f_water_tank_temp",
                name="生活热水实际值",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="R"
            ),
            "t_zone1water_settemp1": DeviceAttribute(
                key="t_zone1water_settemp1",
                name="1温区设置值",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="RW"
            ),
            "t_zone2water_settemp2": DeviceAttribute(
                key="t_zone2water_settemp2",
                name="2温区设置值",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="RW"
            ),
            "t_dhw_temp": DeviceAttribute(
                key="t_dhw_temp",
                name="生活热水设置值",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="RW"
            ),
            "f_power_consumption": DeviceAttribute(
                key="f_power_consumption",
                name="电量累积消耗值",
                attr_type="Number",
                read_write="R"
            ),
        }

    def remove_attribute(self, key: str) -> None:
        """移除指定的属性"""
        attributes = self.attributes
        if key in attributes:
            del attributes[key]



"""Parser for Window AC (008-399) device type."""
from typing import Dict

from .base import BaseDeviceParser, DeviceAttribute

class WindowAC008399Parser(BaseDeviceParser):
    """Parser for Window AC 008-399 device type."""
    
    @property
    def device_type(self) -> str:
        return "008"
        
    @property
    def feature_code(self) -> str:
        return "399"
        
    @property
    def attributes(self) -> Dict[str, DeviceAttribute]:
        return {
            "t_work_mode": DeviceAttribute(
                key="t_work_mode",
                name="设定模式",
                attr_type="Enum",
                step=1,
                value_range="0,1,2,3,5",
                value_map={
                    "0": "送风",
                    "1": "制热",
                    "2": "制冷",
                    "3": "除湿",
                    "5": "E-star"
                },
                read_write="RW"
            ),
            "t_power": DeviceAttribute(
                key="t_power",
                name="开关机",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "关",
                    "1": "开"
                },
                read_write="RW"
            ),
            "t_temp": DeviceAttribute(
                key="t_temp",
                name="设置温度",
                attr_type="Number",
                step=1,
                value_range="16~30,61~86",
                read_write="RW"
            ),
            "t_fan_speed": DeviceAttribute(
                key="t_fan_speed",
                name="设定风速",
                attr_type="Enum",
                step=1,
                value_range="5,7,9",
                value_map={
                    "5": "低风",
                    "7": "中风",
                    "9": "高风"
                },
                read_write="RW"
            ),
            "t_power_consumption": DeviceAttribute(
                key="t_power_consumption",
                name="电量累积消耗值",
                attr_type="Number",
                read_write="R"
            ),
            "t_fan_mute": DeviceAttribute(
                key="t_fan_mute",
                name="设定静音",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "停",
                    "1": "开"
                },
                read_write="RW"
            ),
            "t_super": DeviceAttribute(
                key="t_super",
                name="强力",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "取消",
                    "1": "开启"
                },
                read_write="RW"
            ),
            "t_temp_in": DeviceAttribute(
                key="t_temp_in",
                name="室内温度",
                attr_type="Number",
                read_write="R"
            )
        }



"""Parser for Split AC (009-199) device type."""
from typing import Dict

from .base import BaseDeviceParser, DeviceAttribute

class Humidity007Parser(BaseDeviceParser):
    """Parser for Split AC 009-199 device type."""
    
    @property
    def device_type(self) -> str:
        return "007"
        
    @property
    def feature_code(self) -> str:
        return ""


    @property
    def attributes(self) -> Dict[str, DeviceAttribute]:
        if not hasattr(self, '_attributes'):
            self._attributes = {
                "t_work_mode": DeviceAttribute(
                    key="t_work_mode",
                    name="设定模式",
                    attr_type="Enum",
                    step=1,
                    value_range="1, 0, 15, 5, 6, 16, 3",
                    value_map={
                        "0": "持续",
                        "1": "正常",
                        "2": "自动",
                        "3": "干衣"
                    },
                    read_write="RW"
                ),
                "t_humidity": DeviceAttribute(
                    key="t_humidity",
                    name="设定湿度值",
                    attr_type="Number",
                    step=5,
                    value_range="30~80",
                    read_write="RW"
                ),
                "f_humidity": DeviceAttribute(
                    key="f_humidity",
                    name="实际湿度",
                    attr_type="Number",
                    step=1,
                    value_range="30~90",
                    read_write="R"
                ),
                "t_power": DeviceAttribute(
                    key="t_power",
                    name="开关机",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "关",
                        "1": "开"
                    },
                    read_write="RW"
                ),
                "t_fan_speed": DeviceAttribute(
                    key="t_fan_speed",
                    name="设定风速",
                    attr_type="Enum",
                    step=1,
                    value_range="0,5,6,7,8,9",
                    value_map={
                        "2": "自动",
                        "3": "中风",
                        "1": "高风",
                        "0": "低风"
                    },
                    read_write="RW"
                ),
                "f_power_consumption": DeviceAttribute(
                    key="f_power_consumption",
                    name="电量累积消耗值",
                    attr_type="Number",
                    read_write="R"
                ),
            }
        return self._attributes

    def remove_attribute(self, key: str) -> None:
        """移除指定的属性"""
        if key in self._attributes:
            del self._attributes[key]


"""Base device parser class."""
from abc import ABC, abstractmethod
from typing import Any, Dict, Optional
from dataclasses import dataclass
import logging

_LOGGER = logging.getLogger(__name__)

@dataclass
class DeviceAttribute:
    """Device attribute definition."""
    key: str
    name: str
    attr_type: str
    step: int = 1
    value_range: Optional[str] = None  # Value range
    value_map: Optional[Dict[str, str]] = None  # Value mapping
    read_write: str = "RW"

class BaseDeviceParser(ABC):
    """Base class for device parsers."""
    
    @property
    @abstractmethod
    def device_type(self) -> str:
        """Return device type code."""
        pass
    
    @property
    @abstractmethod
    def feature_code(self) -> str:
        """Return feature code."""
        pass

    @property
    @abstractmethod
    def attributes(self) -> Dict[str, DeviceAttribute]:
        """Return device attributes."""
        pass
    def remove_attribute(self, key: str) -> None:
        """Remove specified attribute."""
        attributes = self.attributes
        if key in attributes:
            del attributes[key]

    def parse_status(self, status: Dict[str, Any]) -> Dict[str, Any]:
        """Parse device status."""
        _LOGGER.debug(
            "Parsing status for device type %s-%s with attributes: %s",
            self.device_type,
            self.feature_code,
            {k: attr.name for k, attr in self.attributes.items()}
        )
        _LOGGER.debug("Raw status: %s", status)
        
        parsed_status = {}
        for key, attr in self.attributes.items():
            if key in status:
                value = status[key]
                try:
                    if attr.value_map and value in attr.value_map:
                        parsed_value = attr.value_map[value]
                        # _LOGGER.debug(
                        #     "Mapped attribute %s (%s) from %s to %s",
                        #     key, attr.name, value, parsed_value
                        # )
                    elif attr.attr_type == "Number":
                        parsed_value = float(value)
                        # _LOGGER.debug(
                        #     "Converted attribute %s (%s) to number: %s",
                        #     key, attr.name, parsed_value
                        # )
                    else:
                        parsed_value = value
                        # _LOGGER.debug(
                        #     "Using raw value for attribute %s (%s): %s",
                        #     key, attr.name, parsed_value
                        # )
                    parsed_status[key] = parsed_value
                except (ValueError, TypeError) as err:
                    _LOGGER.warning(
                        "Failed to parse attribute %s (%s) with value %s: %s",
                        key, attr.name, value, err
                    )
                    continue
        
        _LOGGER.debug("Parsed status: %s", parsed_status)
        return parsed_status
    
    def validate_value(self, key: str, value: Any) -> bool:
        """Validate value for a given attribute."""
        if key not in self.attributes:
            _LOGGER.warning("Attribute %s not found in device type %s-%s", 
                          key, self.device_type, self.feature_code)
            return False
            
        attr = self.attributes[key]
        if attr.read_write == "R":
            _LOGGER.warning("Attribute %s (%s) is read-only", key, attr.name)
            return False
            
        if attr.value_range:
            try:
                min_val, max_val = map(float, attr.value_range.split(","))
                value = float(value)
                valid = min_val <= value <= max_val
                if not valid:
                    _LOGGER.warning(
                        "Value %s for attribute %s (%s) is outside valid range %s-%s",
                        value, key, attr.name, min_val, max_val
                    )
                return valid
            except (ValueError, TypeError) as err:
                _LOGGER.warning(
                    "Failed to validate range for attribute %s (%s): %s",
                    key, attr.name, err
                )
                return False
                
        if attr.value_map:
            valid = str(value) in attr.value_map.keys()
            if not valid:
                _LOGGER.warning(
                    "Value %s for attribute %s (%s) is not in valid map %s",
                    value, key, attr.name, list(attr.value_map.keys())
                )
            return valid
            
        return True



"""Device parsers package."""
from typing import Dict, Type
import logging

from .atw_035_699 import SplitWater035699Parser
from .base import BaseDeviceParser
from .base_bean import BaseBeanParser
from .hum_007 import Humidity007Parser
from .split_ac_009_199 import SplitAC009199Parser
from .window_ac_008_399 import WindowAC008399Parser
from .bean_006_299 import Split006299Parser

_LOGGER = logging.getLogger(__name__)

# Registry of device parsers
DEVICE_PARSERS: Dict[tuple[str, str], Type[BaseDeviceParser]] = {
    ("035", "699"): SplitWater035699Parser,
    ("006", "299"): Split006299Parser,
    ("007", ""): Humidity007Parser,
}


def get_device_parser(device_type: str , feature_code: str) -> Type[BaseDeviceParser]:
    """Get device parser for the given device type."""
    _LOGGER.debug("Getting device parser for type %s", device_type)
    if DEVICE_PARSERS.get((device_type, feature_code)):
        _LOGGER.debug("Triple supply device %s", device_type)
        return DEVICE_PARSERS[(device_type, feature_code)]
    if device_type == "007":
        _LOGGER.debug("Dehumidifier device %s", device_type)
        return Humidity007Parser
    # Pre-defined device type set
    supported_device_types = ["009", "008", "006", "016"]
    if device_type in supported_device_types:
        parser_class = BaseBeanParser
        _LOGGER.debug("Using default parser for device type %s", device_type)
        return parser_class
    else:
        _LOGGER.warning("Unsupported device type: %s", device_type)
        raise ValueError(f"Unsupported device type: {device_type}")




# base_bean.py
from typing import Dict, List

from .base import BaseDeviceParser, DeviceAttribute

class BaseBeanParser(BaseDeviceParser):

    @property
    def device_type(self) -> str:
        return "1"

    @property
    def feature_code(self) -> str:
        return "2"

    def remove_attribute(self, key: str) -> None:
        """Remove specified attribute."""
        if key in self._attributes:
            del self._attributes[key]


    @property
    def attributes(self) -> Dict[str, DeviceAttribute]:
        if not hasattr(self, '_attributes'):
            self._attributes = {
                "t_work_mode": DeviceAttribute(
                    key="t_work_mode",
                    name="设定模式",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1,2,3,4,5",
                    value_map={
                        "0": "送风",
                        "1": "制热",
                        "2": "制冷",
                        "3": "除湿",
                        "4": "自动",
                        "5": "E-star",
                        "8": "标准模式",
                        "9": "ECO模式",
                        "10": "双能热水模式",
                        "11": "双能1模式",
                        "12": "电热热水模式"
                    },
                    read_write="RW"
                ),
                "t_power": DeviceAttribute(
                    key="t_power",
                    name="开关机",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "关",
                        "1": "开"
                    },
                    read_write="RW"
                ),
                "t_temp": DeviceAttribute(
                    key="t_temp",
                    name="设置温度",
                    attr_type="Number",
                    step=1,
                    value_range="16~32,61~90",
                    read_write="RW"
                ),
                "t_fan_speed": DeviceAttribute(
                    key="t_fan_speed",
                    name="设定风速",
                    attr_type="Enum",
                    step=1,
                    value_range="0,5,6,7,8,9",
                    value_map={
                        "2": "低风",
                        "3": "中风",
                        "4": "高风",
                        "0": "自动",
                        "5": "低",
                        "6": "中低",
                        "7": "中",
                        "8": "中高",
                        "9": "高"
                    },
                    read_write="RW"
                ),
                "t_up_down": DeviceAttribute(
                    key="t_up_down",
                    name="上下风",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "取消",
                        "1": "开启"
                    },
                    read_write="RW"
                ),
                "t_temp_type": DeviceAttribute(
                    key="t_temp_type",
                    name="温度单位切换",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "摄氏",
                        "1": "华氏"
                    },
                    read_write="RW"
                ),
                "t_left_right": DeviceAttribute(
                    key="t_left_right",
                    name="左右风",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "取消",
                        "1": "开启"
                    },
                    read_write="RW"
                ),
                "f_power_consumption": DeviceAttribute(
                    key="f_power_consumption",
                    name="电量累积消耗值",
                    attr_type="Number",
                    read_write="R"
                ),
                "t_fan_mute": DeviceAttribute(
                    key="t_fan_mute",
                    name="设定静音",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "停",
                        "1": "开"
                    },
                    read_write="RW"
                ),
                "f_temp_in": DeviceAttribute(
                    key="f_temp_in",
                    name="室内温度",
                    attr_type="Number",
                    read_write="R"
                ),
                "t_8heat": DeviceAttribute(
                    key="t_8heat",
                    name="8°制热",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "关闭",
                        "1": "开启"
                    },
                    read_write="RW"
                ),
                "t_eco": DeviceAttribute(
                    key="t_eco",
                    name="节能模式",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "关闭",
                        "1": "开启"
                    },
                    read_write="RW"
                ),
                "t_humidity": DeviceAttribute(
                    key="t_humidity",
                    name="设定湿度值",
                    attr_type="Number",
                    step=5,
                    value_range="30~80",
                    read_write="RW"
                ),
                "f_humidity": DeviceAttribute(
                    key="f_humidity",
                    name="实际湿度",
                    attr_type="Number",
                    step=1,
                    value_range="30~90",
                    read_write="R"
                ),
                "t_super": DeviceAttribute(
                    key="t_super",
                    name="强力",
                    attr_type="Enum",
                    step=1,
                    value_range="0,1",
                    value_map={
                        "0": "取消",
                        "1": "开启"
                    },
                    read_write="RW"
                )
            }
        return self._attributes

    @attributes.setter
    def attributes(self, value: Dict[str, DeviceAttribute]):
        self._attributes = value



"""Setup script for ConnectLife Cloud API client."""

from setuptools import find_packages, setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Read requirements from pyproject.toml instead
requirements = [
    "aiohttp>=3.8.0",
    "pytz>=2021.1",
]

setup(
    name="connectlife-cloud",
    version="0.3.2",
    author="ConnectLife LLC",
    author_email="support@connectlife.com",
    description="ConnectLife Cloud API client library for Home Assistant integrations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Connectlife-LLC/connectlife-cloud",
    license="MIT",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.8",
    install_requires=requirements,
    extras_require={
        "dev": [
            "pytest>=6.0",
            "pytest-asyncio>=0.18.0",
            "pytest-cov>=2.10.0",
            "black>=21.0.0",
            "isort>=5.0.0",
            "mypy>=0.910",
            "pre-commit>=2.15.0",
        ],
    },
)



"""Parser for Split AC (009-199) device type."""
from typing import Dict

from .base import BaseDeviceParser, DeviceAttribute

class SplitAC009199Parser(BaseDeviceParser):
    """Parser for Split AC 009-199 device type."""
    
    @property
    def device_type(self) -> str:
        return "009"
        
    @property
    def feature_code(self) -> str:
        return "199"
        
    @property
    def attributes(self) -> Dict[str, DeviceAttribute]:
        return {
            "t_work_mode": DeviceAttribute(
                key="t_work_mode",
                name="设定模式",
                attr_type="Enum",
                step=1,
                value_range="0,1,2,3,4",
                value_map={
                    "0": "送风",
                    "1": "制热",
                    "2": "制冷",
                    "3": "除湿",
                    "4": "自动"
                },
                read_write="RW"
            ),
            "t_power": DeviceAttribute(
                key="t_power",
                name="开关机",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "关",
                    "1": "开"
                },
                read_write="RW"
            ),
            "t_temp": DeviceAttribute(
                key="t_temp",
                name="设置温度",
                attr_type="Number",
                step=1,
                value_range="16~32,61~90",
                read_write="RW"
            ),
            "t_fan_speed": DeviceAttribute(
                key="t_fan_speed",
                name="设定风速",
                attr_type="Enum",
                step=1,
                value_range="0,5,6,7,8,9",
                value_map={
                    "0": "自动",
                    "5": "超低",
                    "6": "低",
                    "7": "中",
                    "8": "高",
                    "9": "超高"
                },
                read_write="RW"
            ),
            "t_up_down": DeviceAttribute(
                key="t_up_down",
                name="上下风",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "取消",
                    "1": "开启"
                },
                read_write="RW"
            ),
            "t_left_right": DeviceAttribute(
                key="t_left_right",
                name="左右风",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "取消",
                    "1": "开启"
                },
                read_write="RW"
            ),
            "t_power_consumption": DeviceAttribute(
                key="t_power_consumption",
                name="电量累积消耗值",
                attr_type="Number",
                read_write="R"
            ),
            "t_fan_mute": DeviceAttribute(
                key="t_fan_mute",
                name="设定静音",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "停",
                    "1": "开"
                },
                read_write="RW"
            ),
            "t_super": DeviceAttribute(
                key="t_super",
                name="强力",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "取消",
                    "1": "开启"
                },
                read_write="RW"
            ),
            "t_temp_in": DeviceAttribute(
                key="t_temp_in",
                name="室内温度",
                attr_type="Number",
                read_write="R"
            ),
            "t_8heat": DeviceAttribute(
                key="t_8heat",
                name="8°制热",
                attr_type="Enum",
                step=1,
                value_range="0,1",
                value_map={
                    "0": "关闭",
                    "1": "开启"
                },
                read_write="RW"
            )
        }
